#define bitRead(value, bit) (((value) >> (bit)) & 0x01)
#define bitSet(value, bit) ((value) |= (1UL << (bit)))
#define bitClear(value, bit) ((value) &= ~(1UL << (bit)))
#define bitWrite(value, bit, bitvalue) (bitvalue ? bitSet(value, bit) : bitClear(value, bit))
#include <HX711.h>

HX711::HX711(byte dout, byte pd_sck, byte gain) {
	PD_SCK 	= pd_sck;
	DOUT 	= dout;
	
	pinMode(PD_SCK, OUTPUT);
	pinMode(DOUT, INPUT_PULLUP);

	//Serial.println("hx711 init");
	//Serial.println(PD_SCK);
	//Serial.println(DOUT);
	set_gain(gain);
}

HX711::~HX711() {

}

bool HX711::is_ready() {
	if(digitalRead(DOUT) == LOW)
	{
		////Serial.println("LOW");
	}
	else
	{
		////Serial.println("HIGH");
	}
	return digitalRead(DOUT) == LOW;
}

void HX711::set_gain(byte gain) {
	switch (gain) {
		case 128:		// channel A, gain factor 128
			GAIN = 1;

			break;
		case 64:		// channel A, gain factor 64
			GAIN = 3;
			break;
		case 32:		// channel B, gain factor 32
			GAIN = 2;
			break;
	}
	//Serial.println("GAIN : ");
	//Serial.println(GAIN);
	digitalWrite(PD_SCK, LOW);
	read();
}

long HX711::read() {
	// wait for the chip to become ready
	////Serial.println("ready");
	
	while (!is_ready()); 
	byte data[3];
    
	//Serial.println("bitwrite start");
	// pulse the clock pin 24 times to read the data
	for (byte j = 3; j--;) {
		//Serial.print("byte bit write start : ");
		for (char i = 8; i--;) {
			digitalWrite(PD_SCK, HIGH);
			bitWrite(data[j], i, digitalRead(DOUT));
			digitalWrite(PD_SCK, LOW);
			//Serial.print(data[j]);
			//Serial.print("   ");	
		}
		//Serial.print("byte value ");
		//Serial.print(j);
		//Serial.print(" : ");
	    //Serial.println(data[j]);
	}
    
	
	//Serial.print("set chennel GAIN Value : ");
	//Serial.println(GAIN);		
	for (int i = 0; i < GAIN; i++) {
		digitalWrite(PD_SCK, HIGH);
		digitalWrite(PD_SCK, LOW);
	}

	//Serial.print("set data data[2] value : ");
	data[2] ^= 0x80;
	//Serial.println(data[2]);
	
	////Serial.println("set retur");
	return ((uint32_t) data[2] << 16) | ((uint32_t) data[1] << 8) | (uint32_t) data[0];
}

long HX711::read_average(byte times) {
	long sum = 0;
	//Serial.print("read_average start , times value : ");
    //Serial.println(times);
	for (byte i = 0; i < times; i++) {
		sum += read();
	}
     //Serial.print("read_average end , sum value : ");
    //Serial.println(sum);
	return sum / times;
}

double HX711::get_value(byte times) {
	//Serial.print("get_values start , times, offset value : ");
    //Serial.print(times);
	//Serial.print(" , ");
	//Serial.println(OFFSET);
	return read_average(times) - OFFSET;
}

float HX711::get_units(byte times) {
	//Serial.print("get_units start , times, SCALE value : ");
    //Serial.print(times);
	//Serial.print(" , ");
	//Serial.println(SCALE);
	return get_value(times) / SCALE;
}

void HX711::tare(byte times) {
	//Serial.print("tare start , times value : ");
    //Serial.println(times);
	double sum = read_average(times);
	//Serial.print("tare end , sum value : ");
    //Serial.println(sum);
	
	set_offset(sum);
}

void HX711::set_scale(float scale) {
	SCALE = scale;
	//Serial.print("Scale value : ");
	//Serial.println(SCALE);
}

void HX711::set_offset(long offset) {
	OFFSET = offset;
    //Serial.print("offset value : ");
	//Serial.println(OFFSET);
}

void HX711::power_down() {
	digitalWrite(PD_SCK, LOW);
	digitalWrite(PD_SCK, HIGH);	
}

void HX711::power_up() {
	digitalWrite(PD_SCK, LOW);	
}